// Zoom Tube Boost by Yellow/@GlowingTail
-- what if you could mash to go faster on zoom tubes

/*	SOC entries
 - Lua.zoomtubespeedup		How much speed will increase per input. This script checks for this value to activate.
 - Lua.zoomtubespeedcap		Speed cap of the zoom tube, this must be higher than the actual set speed otherwise it will change the zoom tube's speed.
 - Lua.zoomtubemashing		Cooldown between speed increases in tics, if set to 0 it will automatically increase speed each tic without requiring any input.
*/

local zoomtubeSpeedUp, zoomtubeSpeedCap, mashingCooldown
local remind = false

local function initPlayer(p)
	p.zoomtubeboost = {
		cooldown = 0,
		boostbtn = 2,
		btnstate = {0, 0, 0,
			Accel = 0,
			UpDown = 0,
			LeftRight = 0
		}
	}
end

local btnLookup = {
	['1'] = 1, ['2'] = 2, ['3'] = 3,
	accel = 1, directional = 2, both = 3
}

---epic wanted this at gunpoint so we out here
COM_AddCommand('ztb_button', function(p, input)
	if not p.zoomtubeboost initPlayer(p) end
	local btn = {'Accel', 'Directional', 'Both'}

	if not input
		CONS_Printf(p, '"ztb_button" is set to "\130'..btn[p.zoomtubeboost.boostbtn]..'\128".\nAvailable options: Accel / Directional / Both')
		return
	elseif not btnLookup[input:lower()]
		CONS_Printf(p, '"\130'..input..'\128" is not a possible value for "ztb_button".\nAvailable options: Accel / Directional / Both')
		return
	end

	p.zoomtubeboost.boostbtn = btnLookup[input:lower()]
	CONS_Printf(p, '"ztb_button" has been set to "\130'..btn[p.zoomtubeboost.boostbtn]..'\128".')
end)

local function getZTB(mapHeader)
	if not mapHeader.zoomtubespeedup
		return nil, nil, nil
	end

	return max(mapHeader.zoomtubespeedup, 1) * mapobjectscale, max(mapHeader.zoomtubespeedcap or 4096, 1) * mapobjectscale, max(mapHeader.zoomtubemashing or 10, 0)
end

local function buttonTrackState(p)
	local btnstate = p.zoomtubeboost.btnstate
	local cooldown = p.zoomtubeboost.cooldown

	btnstate.Accel = (p.cmd.buttons & BT_ACCELERATE) and (($ or 0) + 1) or 0

	if p.cmd.buttons & BT_FORWARD or (btnstate.UpDown > 0 and cooldown)
		btnstate.UpDown = max($ + 1, 0)
	elseif p.cmd.buttons & BT_BACKWARD or (btnstate.UpDown < 0 and cooldown)
		btnstate.UpDown = min($ - 1, 0)
	end

	if p.cmd.driftturn > 0 or (btnstate.LeftRight > 0 and cooldown)
		btnstate.LeftRight = max($ + 1, 0)
	elseif p.cmd.driftturn < 0 or (btnstate.LeftRight < 0 and cooldown)
		btnstate.LeftRight = min($ - 1, 0)
	end

	btnstate[1] = btnstate.Accel
	btnstate[2] = min(abs(btnstate.UpDown), abs(btnstate.LeftRight))
	btnstate[3] = btnstate[1] or btnstate[2] or 0
end

addHook('ThinkFrame', do
	zoomtubeSpeedUp, zoomtubeSpeedCap, mashingCooldown = getZTB(mapheaderinfo[gamemap])
	if not zoomtubeSpeedUp return end

	for p in players.iterate do
		if not p.zoomtubeboost
			initPlayer(p)
		end

		if not (p.mo and p.mo.valid and zoomtubeSpeedUp) continue end
		local speedUp, buttonPress = zoomtubeSpeedUp / max(mashingCooldown, 1), p.zoomtubeboost.btnstate[p.zoomtubeboost.boostbtn]

		if (p.mo.tracer and p.mo.tracer.valid and p.mo.tracer.type == MT_TUBEWAYPOINT)
			buttonTrackState(p)

			if p.zoomtubeboost.cooldown > 0 or mashingCooldown == 0
				p.speed = ($ + speedUp <= zoomtubeSpeedCap) and min($ + speedUp, zoomtubeSpeedCap) or zoomtubeSpeedCap
				p.zoomtubeboost.cooldown = max($ - 1, 0)
			elseif p.zoomtubeboost.cooldown <= 0 and ((buttonPress == 1) or p.kartstuff[k_sneakertimer])
				p.zoomtubeboost.cooldown = mashingCooldown
			end
		else
			p.zoomtubeboost.cooldown = mashingCooldown
		end
	end
end)

hud.add(function(v, p)
	if not zoomtubeSpeedUp or splitscreen or mashingCooldown == 0 return end

	local function Animate(loop, speed)
		return leveltime / speed % loop
	end

	local prompts = {'ZTBBTN'..Animate(3, 3), 'ZTBSCK'..Animate(4, 4)}
	local btntype = (p.zoomtubeboost.boostbtn == 3) and (1 + leveltime / 40 % 2) or p.zoomtubeboost.boostbtn
	local instruction = {'\136Mash the accelerate button', '\136Use directional controls'}

	if (p and p.valid and p.mo and p.mo.valid and p.mo.tracer and p.mo.tracer.valid and p.mo.tracer.type == MT_TUBEWAYPOINT)
		v.draw(160, 100, v.cachePatch(prompts[btntype]),V_SNAPTORIGHT|V_SNAPTOBOTTOM)
		if not remind
			chatprintf(p, ((p.zoomtubeboost.boostbtn == 3) and table.concat(instruction, ' \130or ') or instruction[p.zoomtubeboost.boostbtn])..' \130to go faster on Zoom Tubes!', true)
			remind = true
		end
	end
end)